import _filterInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/filter';
import { program } from 'commander';
import { Listr } from 'listr2';
import execa from 'execa';
import semver from 'semver';
import _Promise from '@babel/runtime-corejs3/core-js-stable/promise';
import _startsWithInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/starts-with';
import crypto from 'node:crypto';
import path from 'node:path';
import readline from 'node:readline';
import { CLOUD_IDENTIFIERS } from '@commercetools-frontend/application-config';
import _trimInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/trim';
import _sliceInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/slice';
import _mapInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/map';
import _findInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/find';
import fs from 'node:fs';
import _Object$keys from '@babel/runtime-corejs3/core-js-stable/object/keys';
import _includesInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/includes';
import _Object$values from '@babel/runtime-corejs3/core-js-stable/object/values';
import _Date$now from '@babel/runtime-corejs3/core-js-stable/date/now';
import os from 'node:os';
import _Object$getOwnPropertySymbols from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-symbols';
import _Object$getOwnPropertyDescriptor from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-descriptor';
import _forEachInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/for-each';
import _Object$getOwnPropertyDescriptors from '@babel/runtime-corejs3/core-js-stable/object/get-own-property-descriptors';
import _Object$defineProperties from '@babel/runtime-corejs3/core-js-stable/object/define-properties';
import _Object$defineProperty from '@babel/runtime-corejs3/core-js-stable/object/define-property';
import _defineProperty from '@babel/runtime-corejs3/helpers/esm/defineProperty';
import _slicedToArray from '@babel/runtime-corejs3/helpers/esm/slicedToArray';
import _reduceInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/reduce';
import _Object$entries from '@babel/runtime-corejs3/core-js-stable/object/entries';
import _Object$assign from '@babel/runtime-corejs3/core-js-stable/object/assign';
import _JSON$stringify from '@babel/runtime-corejs3/core-js-stable/json/stringify';
import { transformFileSync, types } from '@babel/core';
import prettier from 'prettier';

var pkgJson = {
	name: "@commercetools-frontend/create-mc-app",
	version: "24.1.0",
	description: "Create Merchant Center applications to quickly get up and running",
	bugs: "https://github.com/commercetools/merchant-center-application-kit/issues",
	repository: {
		type: "git",
		url: "https://github.com/commercetools/merchant-center-application-kit.git",
		directory: "packages/create-mc-app"
	},
	homepage: "https://docs.commercetools.com/merchant-center-customizations",
	keywords: [
		"javascript",
		"frontend",
		"react",
		"toolkit"
	],
	license: "MIT",
	publishConfig: {
		access: "public"
	},
	main: "dist/commercetools-frontend-create-mc-app.cjs.js",
	module: "dist/commercetools-frontend-create-mc-app.esm.js",
	bin: "./bin/cli.js",
	dependencies: {
		"@babel/core": "^7.22.17",
		"@babel/runtime": "^7.22.15",
		"@babel/runtime-corejs3": "^7.22.15",
		"@commercetools-frontend/application-config": "^24.1.0",
		"@types/babel__core": "^7.20.1",
		"@types/semver": "^7.5.1",
		commander: "^13.1.0",
		execa: "5.1.1",
		listr2: "5.0.8",
		prettier: "2.8.8",
		semver: "7.6.2"
	},
	devDependencies: {
		"@tsconfig/node22": "^22.0.0",
		"@types/prettier": "2.7.3"
	},
	engines: {
		node: "18.x || 20.x || >=22.0.0"
	}
};

const applicationTypes = {
  'custom-application': 'custom-application',
  'custom-view': 'custom-view'
};
const availableTemplates = {
  starter: 'starter',
  'starter-typescript': 'starter-typescript'
};

async function getLatestReleaseVersion() {
  const commandResult = await execa.command('npm view @commercetools-frontend/create-mc-app --json', {
    encoding: 'utf-8'
  });
  const packageInfoForTagLatest = JSON.parse(commandResult.stdout);
  return packageInfoForTagLatest.version;
}

function hintOutdatedVersion(currentVersion, releaseVersion) {
  var _context;
  const hasBeenReleastedInLatestTag = semver.gt(releaseVersion, currentVersion);
  const hintNewerVersions = _filterInstanceProperty(_context = [hasBeenReleastedInLatestTag && `${releaseVersion}`]).call(_context, Boolean).join(', ');
  if (hintNewerVersions.length > 0) {
    console.log('');
    console.log(`New version available! ${currentVersion} -> ${hintNewerVersions}`);
    console.log('');
  }
}

const isSemVer = version => /^(v?)([0-9].[0-9].[0-9])+/.test(version);
const doesFileExist = path => {
  try {
    fs.accessSync(path);
    return true;
  } catch (error) {
    return false;
  }
};
const shouldUseYarn = () => {
  try {
    const result = execa.commandSync('yarn --version', {
      stdio: 'ignore'
    });
    return !result.failed;
  } catch (error) {
    return false;
  }
};
const getYarnVersion = () => {
  try {
    var _context;
    const result = execa.commandSync('yarn --version', {
      encoding: 'utf-8'
    });
    return _trimInstanceProperty(_context = result.stdout).call(_context);
  } catch (error) {
    return null;
  }
};
const configureYarn = projectDirectoryPath => {
  const yarnVersion = getYarnVersion();
  if (!yarnVersion) return;

  // Check if Yarn version is 2 or higher. Plug'n'Play was introduced in Yarn v2
  if (semver.gte(yarnVersion, '2.0.0')) {
    // Sanitize and resolve the path
    const normalizedProjectPath = path.resolve(projectDirectoryPath);

    // Create .yarnrc.yml to use node_modules
    const yarnrcPath = path.join(normalizedProjectPath, '.yarnrc.yml');
    const yarnrcContent = 'nodeLinker: "node-modules"';
    fs.writeFileSync(yarnrcPath, yarnrcContent);
  }
};
const getPreferredPackageManager = options => {
  if (options.packageManager) {
    return options.packageManager;
  }
  // Attempt to use yarn (backwards compatibility)
  if (shouldUseYarn()) {
    return 'yarn';
  }
  // Fall back to npm
  return 'npm';
};
const getInstallCommand = options => {
  const packageManager = getPreferredPackageManager(options);
  switch (packageManager) {
    case 'npm':
      return 'npm install --legacy-peer-deps';
    default:
      return `${packageManager} install`;
  }
};
const slugify = name => name.toLowerCase().replace(/_/gi, '-');
const upperFirst = value => value.charAt(0).toUpperCase() + _sliceInstanceProperty(value).call(value, 1);
const wordify = slug => {
  var _context2;
  return _mapInstanceProperty(_context2 = slug.split('-')).call(_context2, word => upperFirst(word)).join(' ');
};
const resolveFilePathByExtension = requestedModule => {
  var _context3;
  const fileExtension = _findInstanceProperty(_context3 = ['.js', '.ts', '.mjs', '.cjs']).call(_context3, ext => {
    const filePath = `${requestedModule}${ext}`;
    return doesFileExist(filePath);
  });
  return `${requestedModule}${fileExtension}`;
};

const throwIfApplicationTypeIsNotSupported = applicationType => {
  var _context;
  const applicationTypesList = _Object$keys(applicationTypes);
  if (!_includesInstanceProperty(_context = _Object$values(applicationTypesList)).call(_context, applicationType)) {
    throw new Error(`The provided application type "${applicationType.toString()}" does not exist. Available types are "${applicationTypesList}". Make sure you are also using the latest version of "@commercetools-frontend/create-mc-app".`);
  }
};
const throwIfTemplateIsNotSupported = templateName => {
  switch (templateName) {
    case availableTemplates.starter:
    case availableTemplates['starter-typescript']:
      break;
    default:
      {
        const templateNamesList = _Object$keys(availableTemplates).toString();
        throw new Error(`The provided template name "${templateName}" does not exist. Available templates are "${templateNamesList}". Make sure you are also using the latest version of "@commercetools-frontend/create-mc-app".`);
      }
  }
};
const throwIfProjectDirectoryExists = (dirName, dirPath) => {
  if (doesFileExist(dirPath)) {
    throw new Error(`A directory named "${dirName}" already exists at this location "${dirPath}". Please choose a different project name or remove the directory, then try running the command again.`);
  }
};
const throwIfTemplateVersionDoesNotExist = (templateName, templateFolderPath, versionToCheck) => {
  if (!doesFileExist(templateFolderPath)) {
    throw new Error(`The downloaded template "${templateName}" does not exist for the given version "${versionToCheck}". Check the releases page if you are looking for a specific version: https://github.com/commercetools/merchant-center-application-kit/releases`);
  }
  // In case the version is semver (usually release tags) we check that
  // the cloned repository contains the template matching the given version
  if (isSemVer(versionToCheck)) {
    const templatePackageJson = JSON.parse(fs.readFileSync(path.join(templateFolderPath, 'package.json'), {
      encoding: 'utf8'
    }));
    const versionAsNumber = versionToCheck.replace('v', '');
    if (templatePackageJson.version !== versionAsNumber) {
      throw new Error(`The downloaded template "${templateName}" does not match the version "${versionAsNumber}", instead got "${templatePackageJson.version}". Check the releases page if you want to provide a specific version: https://github.com/commercetools/merchant-center-application-kit/releases`);
    }
  }
};
const throwIfInitialProjectKeyIsMissing = initialProjectKey => {
  if (!initialProjectKey) {
    throw new Error(`Provide a valid project key that you have access to.`);
  }
};
const throwIfNodeVersionIsNotSupported = (currentNodeVersion, expectedVersionRange) => {
  const hasValidNodeVersion = semver.satisfies(currentNodeVersion, expectedVersionRange);
  if (!hasValidNodeVersion) {
    throw new Error(`You are running Node ${currentNodeVersion} but create-mc-app requires Node ${expectedVersionRange}. Please update your version of Node.`);
  }
};
const throwIfCloudIdentifierIsNotSupported = cloudIdentifier => {
  const allowedCloudIdentifiers = _Object$values(CLOUD_IDENTIFIERS);
  if (!_includesInstanceProperty(allowedCloudIdentifiers).call(allowedCloudIdentifiers, cloudIdentifier)) {
    throw new Error(`The cloud region "${cloudIdentifier}" is not supported. Supported regions are: ${allowedCloudIdentifiers.join(', ')}.`);
  }
};

const question = (rl, value) => new _Promise(resolve => rl.question(value, resolve));
const getEntryPointUriPath = async (rl, options) => {
  if (options.applicationType === applicationTypes['custom-view']) {
    return;
  }
  if (options.entryPointUriPath) {
    return options.entryPointUriPath;
  }
  const randomEntryPointUriPath = `${options.template}-${crypto.randomBytes(3).toString('hex')}`;
  if (options.yes) {
    return randomEntryPointUriPath;
  }
  const answerEntryPointUriPath = await question(rl, `Provide the Custom Application entryPointUriPath (default "${randomEntryPointUriPath}"): `);
  return answerEntryPointUriPath || randomEntryPointUriPath;
};
const getInitialProjectKey = async (rl, options) => {
  if (options.initialProjectKey) {
    return options.initialProjectKey;
  }
  const initialProjectKey = await question(rl, `Provide the initial project key for local development: `);
  throwIfInitialProjectKeyIsMissing(initialProjectKey);
  return initialProjectKey;
};
const getCloudIdentifier = async (rl, options) => {
  if (options.cloudIdentifier) {
    throwIfCloudIdentifierIsNotSupported(options.cloudIdentifier);
    return options.cloudIdentifier;
  }
  if (options.yes) {
    return CLOUD_IDENTIFIERS.GCP_EU;
  }
  const cloudIdentifier = await question(rl, 'Provide the cloudIdentifier (default "gcp-eu"): ');
  if (!cloudIdentifier) {
    return CLOUD_IDENTIFIERS.GCP_EU;
  }
  throwIfCloudIdentifierIsNotSupported(cloudIdentifier);
  return cloudIdentifier;
};
async function processOptions(projectDirectoryName, options) {
  if (!projectDirectoryName) {
    throw new Error('Missing required argument "[project-directory]"');
  }
  const projectDirectoryPath = path.resolve(projectDirectoryName);

  // Parse options
  let tagOrBranchVersion = options.templateVersion || 'main';
  tagOrBranchVersion = isSemVer(tagOrBranchVersion) && !_startsWithInstanceProperty(tagOrBranchVersion).call(tagOrBranchVersion, 'v') ? `v${tagOrBranchVersion}` : tagOrBranchVersion;
  const templateName = options.template;

  // Validate options
  throwIfApplicationTypeIsNotSupported(options.applicationType);
  throwIfProjectDirectoryExists(projectDirectoryName, projectDirectoryPath);
  throwIfTemplateIsNotSupported(templateName);

  // Read prompts
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  });
  const entryPointUriPath = await getEntryPointUriPath(rl, options);
  const initialProjectKey = await getInitialProjectKey(rl, options);
  const cloudIdentifier = await getCloudIdentifier(rl, options);
  rl.close();
  return {
    applicationType: options.applicationType,
    projectDirectoryName,
    projectDirectoryPath,
    templateName,
    tagOrBranchVersion,
    entryPointUriPath,
    initialProjectKey,
    cloudIdentifier,
    packageManager: options.packageManager
  };
}

const filesToBeRemoved = ['CHANGELOG.md'];
function downloadTemplate(options) {
  return {
    title: 'Downloading template',
    task: () => {
      const tmpDir = os.tmpdir();
      const tmpFolderNameForClonedRepository = ['merchant-center-application-kit', '--', options.tagOrBranchVersion, '--', _Date$now().toString()].join('');
      const clonedRepositoryPath = path.join(tmpDir, tmpFolderNameForClonedRepository);
      const templateFolderPath = path.join(clonedRepositoryPath, options.applicationType === applicationTypes['custom-view'] ? 'custom-views-templates' : 'application-templates', options.templateName);
      return new Listr([{
        title: `Cloning repository using branch ${options.tagOrBranchVersion}`,
        task: async () => {
          // Shallow clone repository
          const result = await execa('git', ['clone', `--branch=${options.tagOrBranchVersion}`, '--depth=1', 'https://github.com/commercetools/merchant-center-application-kit.git', tmpFolderNameForClonedRepository], {
            cwd: tmpDir,
            encoding: 'utf-8'
          });
          if (result.failed) {
            throw new Error(result.stderr);
          }
          throwIfTemplateVersionDoesNotExist(options.templateName, clonedRepositoryPath, options.tagOrBranchVersion);
          return result.stdout;
        }
      }, {
        title: `Copying template ${options.templateName} into project directory ${options.projectDirectoryPath}`,
        task: async () => {
          const command = process.platform === 'win32' || process.platform === 'cygwin' ? 'move' : 'mv';
          const result = await execa(command, [templateFolderPath,
          // Wrap folder path in quotes to avoid issues with empty spaces in the folder names.
          options.projectDirectoryPath], {
            encoding: 'utf-8'
          });
          if (result.failed) {
            throw new Error(result.stderr);
          }
          const templatePackageJsonPath = path.join(options.projectDirectoryPath, 'package.json');
          if (!doesFileExist(templatePackageJsonPath)) {
            throw new Error(`Unable to verify that the template application has a package.json at "${templatePackageJsonPath}"`);
          }
          return result.stdout;
        }
      }, {
        title: `Cleaning up project directory`,
        task: async () => {
          const command = process.platform === 'win32' || process.platform === 'cygwin' ? 'del' : 'rm';
          const result = await execa(command, _mapInstanceProperty(filesToBeRemoved).call(filesToBeRemoved, filePath => path.join(options.projectDirectoryPath, filePath)), {
            encoding: 'utf-8'
          });
          if (result.failed) {
            throw new Error(result.stderr);
          }
          return result.stdout;
        }
      }]);
    }
  };
}

function installDependencies(options) {
  return {
    title: 'Installing dependencies (this might take a while)',
    task: () => {
      const packageManager = getPreferredPackageManager(options);

      // Only configure Yarn if it's the selected package manager
      if (packageManager === 'yarn') {
        configureYarn(options.projectDirectoryPath);
      }
      const installCommand = getInstallCommand(options);

      // TODO: we could check for min yarn/npm versions
      // See https://github.com/facebook/create-react-app/blob/0f4781e8507249ce29a9ac1409fece67c1a53c38/packages/create-react-app/createReactApp.js#L225-L254
      return execa.command(installCommand, {
        cwd: options.projectDirectoryPath,
        encoding: 'utf-8'
      });
    }
  };
}

function ownKeys(e, r) { var t = _Object$keys(e); if (_Object$getOwnPropertySymbols) { var o = _Object$getOwnPropertySymbols(e); r && (o = _filterInstanceProperty(o).call(o, function (r) { return _Object$getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var _context2, _context3; var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? _forEachInstanceProperty(_context2 = ownKeys(Object(t), !0)).call(_context2, function (r) { _defineProperty(e, r, t[r]); }) : _Object$getOwnPropertyDescriptors ? _Object$defineProperties(e, _Object$getOwnPropertyDescriptors(t)) : _forEachInstanceProperty(_context3 = ownKeys(Object(t))).call(_context3, function (r) { _Object$defineProperty(e, r, _Object$getOwnPropertyDescriptor(t, r)); }); } return e; }
const replaceApplicationKitPackageVersionWith = function (releaseVersion) {
  var _context;
  let dependencies = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return _reduceInstanceProperty(_context = _Object$entries(dependencies)).call(_context, (updatedDependencies, _ref) => {
    let _ref2 = _slicedToArray(_ref, 2),
      dependencyName = _ref2[0],
      dependencyVersion = _ref2[1];
    const updatedVersion = dependencyVersion === 'workspace:*' ? releaseVersion : dependencyVersion;
    return _objectSpread(_objectSpread({}, updatedDependencies), {}, {
      [dependencyName]: updatedVersion
    });
  }, {});
};
function updatePackageJson(options, releaseVersion) {
  return {
    title: 'Updating package.json',
    task: () => {
      const packageJsonPath = path.join(options.projectDirectoryPath, 'package.json');
      const appPackageJson = JSON.parse(fs.readFileSync(packageJsonPath, {
        encoding: 'utf8'
      }));
      const packageManager = getPreferredPackageManager(options);
      const updatedAppPackageJson = _Object$assign({}, appPackageJson, {
        version: '1.0.0',
        // Given the package name is derived from the `projectDirectoryName`
        // the latter needs to be sanitised to have a ensure a valid package name.
        // The `projectDirectoryName` should not have restrictions (e.g. no `_`)
        // as a result the package name potentially needs to be altered when derived.
        name: slugify(options.projectDirectoryName),
        description: '',
        // Replace the package versions with the `workspace:` syntax to the real version.
        dependencies: replaceApplicationKitPackageVersionWith(releaseVersion, appPackageJson.dependencies),
        devDependencies: replaceApplicationKitPackageVersionWith(releaseVersion, appPackageJson.devDependencies),
        scripts: _objectSpread(_objectSpread({}, appPackageJson.scripts), {}, {
          'start:prod:local': appPackageJson.scripts['start:prod:local'].replace('yarn', packageManager)
        })
      });
      fs.writeFileSync(packageJsonPath, _JSON$stringify(updatedAppPackageJson, null, 2) + os.EOL, {
        encoding: 'utf8'
      });
    }
  };
}

function replaceApplicationInfoInApplicationConfig(filePath, options) {
  const result = transformFileSync(filePath, {
    plugins: [function replaceConfig() {
      const appName = wordify(options.entryPointUriPath ?? options.projectDirectoryName);
      return {
        visitor: {
          Identifier(nodePath) {
            if (nodePath.isIdentifier({
              name: 'name'
            }) && nodePath.parent.type === 'ObjectProperty') {
              nodePath.parent.value = types.stringLiteral(appName);
            }
            if (nodePath.isIdentifier({
              name: 'initialProjectKey'
            }) && nodePath.parent.type === 'ObjectProperty') {
              nodePath.parent.value = types.stringLiteral(options.initialProjectKey);
            }
            if (nodePath.isIdentifier({
              name: 'cloudIdentifier'
            }) && nodePath.parent.type === 'ObjectProperty') {
              nodePath.parent.value = types.stringLiteral(options.cloudIdentifier);
            }
            if (options.applicationType === applicationTypes['custom-application'] && nodePath.isIdentifier({
              name: 'defaultLabel'
            })) {
              const isMainMenuLinkParent = nodePath.findParent(parentPath => parentPath.isIdentifier({
                name: 'mainMenuLink'
              }));
              if (isMainMenuLinkParent && nodePath.parent.type === 'ObjectProperty') {
                nodePath.parent.value = types.stringLiteral(appName);
              }
            }
          }
        }
      };
    }],
    retainLines: true
  });
  if (result?.code) {
    const prettierConfig = prettier.resolveConfig.sync(options.projectDirectoryPath);
    const formattedData = prettier.format(result.code + os.EOL, prettierConfig ?? undefined);
    fs.writeFileSync(filePath, formattedData, {
      encoding: 'utf8'
    });
  }
}
function getApplicationConfigName(options) {
  switch (options.applicationType) {
    case applicationTypes['custom-application']:
      return 'custom-application-config';
    case applicationTypes['custom-view']:
      return 'custom-view-config';
    default:
      throw new Error(`Unknown application type ${options.applicationType}`);
  }
}
function updateApplicationConfig(options) {
  return {
    title: 'Updating application config file',
    task: () => {
      const configPath = resolveFilePathByExtension(path.join(options.projectDirectoryPath, getApplicationConfigName(options)));
      replaceApplicationInfoInApplicationConfig(configPath, options);
    }
  };
}

function replaceEntryPointUriPathInConstants(filePath, options) {
  const result = transformFileSync(filePath, {
    plugins: [function replaceConstants() {
      return {
        visitor: {
          VariableDeclarator(nodePath) {
            if (nodePath.node.id.type === 'Identifier' && nodePath.node.id.name === 'entryPointUriPath') {
              nodePath.node.init = types.stringLiteral(options.entryPointUriPath);
            }
          }
        }
      };
    }],
    retainLines: true
  });
  if (result?.code) {
    const prettierConfig = prettier.resolveConfig.sync(options.projectDirectoryPath);
    const formattedData = prettier.format(result.code + os.EOL, prettierConfig ?? undefined);
    fs.writeFileSync(filePath, formattedData, {
      encoding: 'utf8'
    });
  }
}
function updateApplicationConstants(options) {
  return {
    title: 'Updating application constants',
    enabled: options.applicationType === applicationTypes['custom-application'],
    task: () => {
      const applicationConstantsPath = resolveFilePathByExtension(path.join(options.projectDirectoryPath, 'src/constants'));
      replaceEntryPointUriPathInConstants(applicationConstantsPath, options);
    }
  };
}

throwIfNodeVersionIsNotSupported(process.versions.node, pkgJson.engines.node);
program.name('create-mc-app').description('CLI to create a new Merchant Center customizations project based on the pre-defined templates.').version(pkgJson.version);

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
process.on('unhandledRejection', err => {
  throw err;
});
const messagesByApplicationType = {
  [applicationTypes['custom-application']]: {
    featureName: 'Custom Application'
  },
  [applicationTypes['custom-view']]: {
    featureName: 'Custom View'
  }
};
const run = () => {
  // Default command
  program.argument('<project-directory>').option('--application-type <type>', '(optional) The type of the application to create: custom-application (default) or custom-view.', applicationTypes['custom-application']).option('--template <name>', '(optional) The name of the template to install.', availableTemplates.starter).option('--template-version <version>', '(optional) The version of the template to install (either a git tag or a git branch of the "commercetools/merchant-center-application-kit" repository).', 'main').option('--skip-install', '(optional) Skip installing the dependencies after cloning the template.', false).option('--yes', '(optional) If set, the prompt options with default values will be skipped.', false).option('--entry-point-uri-path <value>', '(optional) The version of the template to install. (default: starter-<hash>)').option('--initial-project-key <value>', '(optional) A commercetools project key used for the initial login in development. By default, the value is prompted in the terminal.').option('--cloud-identifier <value>', '(optional) Cloud region identifier. By default, the value is prompted in the terminal').option('--package-manager <value>', '(optional) The preferred package manager to use: npm, yarn, pnpm.').action(async (projectDirectory, options) => {
    var _context;
    const releaseVersion = await getLatestReleaseVersion();
    hintOutdatedVersion(pkgJson.version, releaseVersion);
    const taskOptions = await processOptions(projectDirectory, options);
    const messages = messagesByApplicationType[taskOptions.applicationType];
    console.log('');
    console.log('Documentation available at https://docs.commercetools.com/merchant-center-customizations/');
    console.log('');
    const shouldInstallDependencies = !options.skipInstall ||
    // TODO: remove once we manage to ensure the package manager is installed, for example via Corepack.
    options.packageManager === 'pnpm';
    const taskList = new Listr(_filterInstanceProperty(_context = [downloadTemplate(taskOptions), updatePackageJson(taskOptions, releaseVersion), updateApplicationConfig(taskOptions), updateApplicationConstants(taskOptions), shouldInstallDependencies && installDependencies(taskOptions)]).call(_context, Boolean));
    await taskList.run();
    const packageManager = getPreferredPackageManager(taskOptions);
    console.log('');
    console.log(`🎉 🎉 🎉 The ${messages.featureName} has been created in the "${taskOptions.projectDirectoryName}" folder.`);
    console.log('');
    console.log(`To get started:`);
    console.log(`$ cd ${taskOptions.projectDirectoryName}`);
    if (!shouldInstallDependencies) {
      console.log(`$ ${packageManager} install`);
    }
    console.log(`$ ${packageManager} start`);
    console.log('');
    console.log(`Visit https://docs.commercetools.com/merchant-center-customizations/ for more info about developing ${messages.featureName}. Enjoy 🚀`);
  });
  program.parse();
};

export { run as cli };
